==> cloud-4.1.0.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/pages/EntityCreatePage.tsx <==
import Button from 'bootstrap3-components/Button';
import Col from 'bootstrap3-components/Col';
import FluidContainer from 'bootstrap3-components/FluidContainer';
import Form from 'bootstrap3-components/Form';
import GlyphIcon from 'bootstrap3-components/GlyphIcon';
import Row from 'bootstrap3-components/Row';
import ENTITY_FORM_LIST from 'constant/form_template/entity_form_template';
import EntityFormTemplate from 'model/EntityFormTemplate';
import EntityFormLabel from 'molecules/EntityFormLabel';
import EntityFormPanel from 'organisms/EntityFormPanel';
import MenuBar from 'organisms/MenuBar';
import { useEffect, useState } from 'react';
import { useTranslation } from 'react-i18next';
import { useHistory, useParams } from 'react-router-dom';
import { formDataToSendData } from 'service/utility';

const EntityCreatePageImpl = ({ cloudContext, entityFormTemplate, entityName }: {
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
  entityName: string,
}) => {
  const [formData, setFormData] = useState<Record<string, any>>({});
  const { t } = useTranslation();
  const history = useHistory();

  // Initialize form data.
  useEffect(() => {
    const newFormData: Record<string, any> = {};
    entityFormTemplate.entityRecords.forEach((entityRecord) => {
      if (entityRecord.type !== 'label') {
        entityRecord.keyValueRecords.forEach((keyValueRecord) => {
          if (keyValueRecord.type === 'time') {
            newFormData[keyValueRecord.hourName] = keyValueRecord.defaultValue[0];
            newFormData[keyValueRecord.minuteName] = keyValueRecord.defaultValue[1];
          } else if (keyValueRecord.type === 'datetime') {
            // The 'datetime' type is read-only so do not include it in the form data.
          } else {
            newFormData[keyValueRecord.name] = keyValueRecord.defaultValue;
          }
        });
      }
    });
    setFormData(newFormData);
  }, [entityFormTemplate]);

  useEffect(() => {
    console.log(formData);
  }, [formData]);

  /**
   * Save entiy's data.
   */
  const save = async () => {
    const url = `/cloud_dashboard/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityFormTemplate.cloudServiceProvider}_${entityName}/create`;

    const response = await fetch(url, {
      method: 'POST',
      body: formDataToSendData(formData),
    });
    if (!response.ok) {
      console.log('Create Request : No');
      console.error('Create failed.');
      console.error(await response.json());
      window.alert('Create failed.');
    } else {
      console.log('Create Request : Yes');
      console.log(await response.json());

      history.push(`/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityName}`);
    }
  }

  return <>
    <MenuBar
      cloudServiceProvider={entityFormTemplate.cloudServiceProvider}
      cloudContext={cloudContext} />
    <FluidContainer className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            {
              entityFormTemplate.entityRecords.map((entityRecord) => {
                switch (entityRecord.type) {
                  case 'panel':
                    return <EntityFormPanel
                      cloudServiceProvider={entityFormTemplate.cloudServiceProvider}
                      cloudContext={cloudContext}
                      entityName={entityName}
                      actionType={entityFormTemplate.actionType}
                      entityRecord={entityRecord}
                      formData={formData}
                      setFormData={setFormData} />;
                  case 'label':
                    return <EntityFormLabel label={entityRecord.text} entityName={entityName} />;
                  default:
                    return <></>;
                }
              })
            }
            <Button className="icon-before btn-success" onClick={save}>
              <GlyphIcon iconName="ok" />{' ' + t('Save')}
            </Button>
          </Form>
        </Col>
      </Row>
    </FluidContainer>
  </>;
}

const EntityCreatePage = () => {
  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  const entityFormTemplate = ENTITY_FORM_LIST.filter((template) => {
    return template.cloudServiceProvider === params.cloudServiceProvider
      && (template.entityName === params.entityName || template.entityName === '')
      && template.actionType === 'create'
  });

  if (entityFormTemplate.length >= 1) {
    return <EntityCreatePageImpl
      cloudContext={params.cloudContext}
      entityFormTemplate={entityFormTemplate[0]}
      entityName={params.entityName} />;
  }

  // Default Page
  return <>
    <FluidContainer className="px-0">
      <Row>
        <Col>
          <span>EntityCxudPage</span><br />
          <span>cloudServiceProvider={params.cloudServiceProvider}</span><br />
          <span>cloudContext={params.cloudContext}</span><br />
          <span>entityName={params.entityName}</span><br />
          <span>action=create</span>
        </Col>
      </Row>
    </FluidContainer>
  </>;
}

export default EntityCreatePage;

==> cloud-4.2.0.tar.gz: cloud/modules/cloud_dashboard/cloud_dashboard/src/pages/EntityCreatePage.tsx <==
import ENTITY_FORM_LIST from 'constant/form_template/entity_form_template';
import EntityFormTemplate from 'model/EntityFormTemplate';
import EntityFormLabel from 'molecules/EntityFormLabel';
import EntityFormPanel from 'organisms/EntityFormPanel';
import SideBar from 'organisms/SideBar';
import { useEffect, useState } from 'react';
import { Button, Col, Container, Form, Row } from 'react-bootstrap';
import { useTranslation } from 'react-i18next';
import { BsCheckLg } from 'react-icons/bs';
import { useHistory, useParams } from 'react-router-dom';
import { formDataToSendData } from 'service/utility';

const EntityCreatePageImpl = ({ cloudContext, entityFormTemplate, entityName }: {
  cloudContext: string,
  entityFormTemplate: EntityFormTemplate,
  entityName: string,
}) => {
  const [formData, setFormData] = useState<Record<string, any>>({});
  const { t } = useTranslation();
  const history = useHistory();

  // Initialize form data.
  useEffect(() => {
    const newFormData: Record<string, any> = {};
    entityFormTemplate.entityRecords.forEach((entityRecord) => {
      if (entityRecord.type === 'panel') {
        entityRecord.keyValueRecords.forEach((keyValueRecord) => {
          if (keyValueRecord.type === 'time') {
            newFormData[keyValueRecord.hourName] = keyValueRecord.defaultValue[0];
            newFormData[keyValueRecord.minuteName] = keyValueRecord.defaultValue[1];
          } else if (keyValueRecord.type === 'datetime') {
            // The 'datetime' type is read-only so do not include it in the form data.
          } else {
            newFormData[keyValueRecord.name] = keyValueRecord.defaultValue;
          }
        });
      }
    });
    setFormData(newFormData);
  }, [entityFormTemplate]);

  useEffect(() => {
    console.log(formData);
  }, [formData]);

  /**
   * Save entiy's data.
   */
  const save = async () => {
    const url = `/cloud_dashboard/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityFormTemplate.cloudServiceProvider}_${entityName}/create`;

    const response = await fetch(url, {
      method: 'POST',
      body: formDataToSendData(formData),
    });
    if (!response.ok) {
      console.log('Create Request : No');
      console.error('Create failed.');
      console.error(await response.json());
      window.alert('Create failed.');
    } else {
      console.log('Create Request : Yes');
      console.log(await response.json());

      history.push(`/${entityFormTemplate.cloudServiceProvider}/${cloudContext}/${entityName}`);
    }
  }

  return <>
    <SideBar />
    <Container fluid className="px-0">
      <Row className="mx-0">
        <Col>
          <Form>
            {
              entityFormTemplate.entityRecords.map((entityRecord) => {
                switch (entityRecord.type) {
                  case 'panel':
                    return <EntityFormPanel
                      cloudServiceProvider={entityFormTemplate.cloudServiceProvider}
                      cloudContext={cloudContext}
                      entityName={entityName}
                      actionType={entityFormTemplate.actionType}
                      entityRecord={entityRecord}
                      formData={formData}
                      setFormData={setFormData} />;
                  case 'label':
                    return <EntityFormLabel label={entityRecord.text} entityName={entityName} />;
                  default:
                    return <></>;
                }
              })
            }
            <Button className="ripple-effect" onClick={save}>
              <BsCheckLg />{' ' + t(
                entityFormTemplate.submitButtonLabel
                  ? entityFormTemplate.submitButtonLabel
                  : t('Save')
              )}
            </Button>
          </Form>
        </Col>
      </Row>
    </Container>
  </>;
}

const EntityCreatePage = () => {
  const params: {
    cloudServiceProvider: string,
    cloudContext: string,
    entityName: string,
  } = useParams();

  const entityFormTemplate = ENTITY_FORM_LIST.filter((template) => {
    return template.cloudServiceProvider === params.cloudServiceProvider
      && (template.entityName === params.entityName || template.entityName === '')
      && template.actionType === 'create'
  });

  if (entityFormTemplate.length >= 1) {
    return <EntityCreatePageImpl
      cloudContext={params.cloudContext}
      entityFormTemplate={entityFormTemplate[0]}
      entityName={params.entityName} />;
  }

  // Default Page
  return <>
    <SideBar />
    <Container fluid className="px-0">
      <Row>
        <Col>
          <span>EntityCxudPage</span><br />
          <span>cloudServiceProvider={params.cloudServiceProvider}</span><br />
          <span>cloudContext={params.cloudContext}</span><br />
          <span>entityName={params.entityName}</span><br />
          <span>action=create</span>
        </Col>
      </Row>
    </Container>
  </>;
}

export default EntityCreatePage;
